@@ -18,7 +18,9 @@ use File::Path ();
 use Cwd ();
 use Config;
 
-our $VERSION = '0.912';
+our $VERSION = '0.913';
+our $GIT_DESCRIBE;
+our $GIT_URL;
 
 use constant WIN32 => $^O eq 'MSWin32';
 
@@ -75,10 +77,8 @@ sub parse_options {
     };
     GetOptions
         "L|local-lib-contained=s" => \($self->{local_lib}),
-        "V|version" => sub { $self->cmd_version; exit },
         "color!" => \($self->{color}),
         "g|global" => \($self->{global}),
-        "h|help" => sub { $self->cmd_help },
         "mirror=s@" => \@mirror,
         "v|verbose" => \($self->{verbose}),
         "w|workers=i" => \($self->{workers}),
@@ -135,7 +135,26 @@ sub parse_options {
     $App::cpm::Logger::COLOR = 1 if $self->{color};
     $App::cpm::Logger::VERBOSE = 1 if $self->{verbose};
     $App::cpm::Logger::SHOW_PROGRESS = 1 if $self->{show_progress};
-    $self->{argv} = \@ARGV;
+
+    if (@ARGV && $ARGV[0] eq "-") {
+        $self->{argv} = $self->read_argv_from_stdin;
+        $self->{cpanfile} = undef;
+    } else {
+        $self->{argv} = \@ARGV;
+    }
+}
+
+sub read_argv_from_stdin {
+    my $self = shift;
+    my @argv;
+    while (my $line = <STDIN>) {
+        next if $line !~ /\S/;
+        next if $line =~ /^\s*#/;
+        $line =~ s/^\s*//;
+        $line =~ s/\s*$//;
+        push @argv, split /\s+/, $line;
+    }
+    return \@argv;
 }
 
 sub _inc {
@@ -198,32 +217,22 @@ sub cmd_help {
 
 sub cmd_version {
     print "cpm $VERSION ($0)\n";
-}
-
-sub cmd_exec {
-    my ($self, @argv) = @_;
-    warn "WARN: exec subcommand is deprecated.\n"
-        ."WARN: Please use `perl -I\$PWD/local/lib/perl5 ...` or `env PERL5LIB=\$PWD/local/lib/perl5 perl ...`.\n";
-    my $local_lib = $self->maybe_abs($self->{local_lib});
-    if (-d "$local_lib/lib/perl5") {
-        $ENV{PERL5LIB} = "$local_lib/lib/perl5"
-                       . ($ENV{PERL5LIB} ? ":$ENV{PERL5LIB}" : "");
+    if ($GIT_DESCRIBE && $GIT_URL) {
+        print "This is a self-contained version, $GIT_DESCRIBE ($GIT_URL)\n";
     }
-    if (-d "$local_lib/bin") {
-        $ENV{PATH} = "$local_lib/bin:$ENV{PATH}";
-    }
-    exec @argv;
-    exit 255;
+    return 0;
 }
 
 sub cmd_install {
     my $self = shift;
-    die "Need arguments or cpanfile.\n" if !@{$self->{argv}} && !-f $self->{cpanfile};
+    die "Need arguments or cpanfile.\n"
+        if !@{$self->{argv}} && (!$self->{cpanfile} || !-f $self->{cpanfile});
 
     File::Path::mkpath($self->{home}) unless -d $self->{home};
     my $logger = App::cpm::Logger::File->new("$self->{home}/build.log.@{[time]}");
     $logger->symlink_to("$self->{home}/build.log");
     $logger->log("Running cpm $VERSION ($0) on perl $Config{version} built for $Config{archname} ($^X)");
+    $logger->log("This is a self-contained version, $GIT_DESCRIBE ($GIT_URL)") if $GIT_DESCRIBE && $GIT_URL;
     $logger->log("Command line arguments are: @ARGV");
 
     my $master = App::cpm::Master->new(
@@ -282,7 +291,8 @@ sub cmd_install {
                 App::cpm::Logger->log(result => "FAIL", type => $type, message => $_) for @{$fail->{$type}};
             }
             print STDERR "\r" if $self->{show_progress};
-            warn sprintf "%d distribution installed.\n", $master->installed_distributions;
+            warn sprintf "%d distribution%s installed.\n",
+                $master->installed_distributions, $master->installed_distributions > 1 ? "s" : "";
             warn "See $self->{home}/build.log for details.\n";
             return 1;
         }
@@ -299,7 +309,8 @@ sub cmd_install {
         }
     }
     print STDERR "\r" if $self->{show_progress};
-    warn sprintf "%d distribution installed.\n", $master->installed_distributions;
+    warn sprintf "%d distribution%s installed.\n",
+        $master->installed_distributions, $master->installed_distributions > 1 ? "s" : "";
     $self->cleanup;
 
     if ($fail) {
